/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

// this needs to match the list of pref font.default.xx entries listed in all.js!

FONT_PREF_LANG(Western, "x-western", x_western),
FONT_PREF_LANG(Japanese, "ja", Japanese),
FONT_PREF_LANG(ChineseTW, "zh-TW", Taiwanese),
FONT_PREF_LANG(ChineseCN, "zh-CN", Chinese),
FONT_PREF_LANG(ChineseHK, "zh-HK", HongKongChinese),
FONT_PREF_LANG(Korean, "ko", ko),
FONT_PREF_LANG(Cyrillic, "x-cyrillic", x_cyrillic),
FONT_PREF_LANG(Greek, "el", el),
FONT_PREF_LANG(Thai, "th", th),
FONT_PREF_LANG(Hebrew, "he", he),
FONT_PREF_LANG(Arabic, "ar", ar),
FONT_PREF_LANG(Devanagari, "x-devanagari", x_devanagari),
FONT_PREF_LANG(Tamil, "x-tamil", x_tamil),
FONT_PREF_LANG(Armenian, "x-armn", x_armn),
FONT_PREF_LANG(Bengali, "x-beng", x_beng),
FONT_PREF_LANG(Canadian, "x-cans", x_cans),
FONT_PREF_LANG(Ethiopic, "x-ethi", x_ethi),
FONT_PREF_LANG(Georgian, "x-geor", x_geor),
FONT_PREF_LANG(Gujarati, "x-gujr", x_gujr),
FONT_PREF_LANG(Gurmukhi, "x-guru", x_guru),
FONT_PREF_LANG(Khmer, "x-khmr", x_khmr),
FONT_PREF_LANG(Malayalam, "x-mlym", x_mlym),
FONT_PREF_LANG(Mathematics, "x-math", x_math),
FONT_PREF_LANG(Oriya, "x-orya", x_orya),
FONT_PREF_LANG(Telugu, "x-telu", x_telu),
FONT_PREF_LANG(Kannada, "x-knda", x_knda),
FONT_PREF_LANG(Sinhala, "x-sinh", x_sinh),
FONT_PREF_LANG(Tibetan, "x-tibt", x_tibt),
FONT_PREF_LANG(Others, "x-unicode", Unicode)
