/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef _FREEBL_H_
#define _FREEBL_H_

/* point encoding type */
typedef enum {
    ECPoint_Uncompressed,
    ECPoint_XOnly
} ECPointEncoding;

#endif /* _FREEBL_H_ */
