/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIMacShellService.idl
 */

#ifndef __gen_nsIMacShellService_h__
#define __gen_nsIMacShellService_h__


#ifndef __gen_nsIShellService_h__
#include "nsIShellService.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIMacShellService */
#define NS_IMACSHELLSERVICE_IID_STR "387fdc80-0077-4b60-a0d9-d9e80a83ba64"

#define NS_IMACSHELLSERVICE_IID \
  {0x387fdc80, 0x0077, 0x4b60, \
    { 0xa0, 0xd9, 0xd9, 0xe8, 0x0a, 0x83, 0xba, 0x64 }}

class NS_NO_VTABLE nsIMacShellService : public nsIShellService {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMACSHELLSERVICE_IID)

  enum {
    APPLICATION_KEYCHAIN_ACCESS = 2,
    APPLICATION_NETWORK = 3,
    APPLICATION_DESKTOP = 4
  };

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIMacShellService, NS_IMACSHELLSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMACSHELLSERVICE \

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIMACSHELLSERVICE \

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMACSHELLSERVICE(_to) \

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMACSHELLSERVICE(_to) \

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMacShellService : public nsIMacShellService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMACSHELLSERVICE

  nsMacShellService();

private:
  ~nsMacShellService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsMacShellService, nsIMacShellService)

nsMacShellService::nsMacShellService()
{
  /* member initializers and constructor code */
}

nsMacShellService::~nsMacShellService()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMacShellService_h__ */
