#define MOZ_SOURCE_STAMP ea82b5e20cbbd103f8fa65f0df0386ee4135cc47
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/ea82b5e20cbbd103f8fa65f0df0386ee4135cc47
