/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIHttpHeaderVisitor;

%{C++
namespace mozilla {
namespace net {
class nsHttpResponseHead;
}
}
%}

[ptr] native nsHttpResponseHeadPtr(mozilla::net::nsHttpResponseHead);

/**
 * nsIResponseHeadProvider
 */
[scriptable, builtinclass, uuid(cd0d0804-2e0c-4bff-aa0a-78a3e3159b69)]
interface nsIResponseHeadProvider : nsISupports
{
  /**
   * Returns a pointer to a nsHttpResponseHead. May return null.
   */
  [notxpcom] nsHttpResponseHeadPtr GetResponseHead();

  /**
   * May be used to iterate through the response headers
   */
  void visitResponseHeaders(in nsIHttpHeaderVisitor aVisitor);
};
